package org.codehaus.xfire.message;

import java.util.Calendar;
import java.util.Date;

import javax.xml.namespace.QName;
import javax.xml.stream.XMLStreamReader;

/**
 * A MessageReader. You must call getNextChildReader() until hasMoreChildReaders()
 * returns false.
 * 
 * @author <a href="mailto:dan@envoisolutions.com">Dan Diephouse</a>
 */
public interface MessageReader
{
    public String getValue();

    public String getValue(String ns, String attribute);

    public Calendar getValueAsCalendar();
    
    public Calendar getValueAsCalendar(String ns, String attribute);

    public int getValueAsInt();

    public int getValueAsInt(String ns, String attribute);

    public long getValueAsLong();

    public long getValueAsLong(String ns, String attribute);

    public double getValueAsDouble();

    public double getValueAsDouble(String ns, String attribute);

    public float getValueAsFloat();

    public float getValueAsFloat(String ns, String attribute);

	public boolean getValueAsBoolean();

	public boolean getValueAsBoolean(String ns, String attribute);

    public Date getValueAsDate();
    
    public Date getValueAsDate(String ns, String attribute);
    
    //public Date getValueAsTime();
    
    public Date getValueAsDateTime();
    
    public Date getValueAsDateTime(String ns, String attribute);
    
    public boolean hasMoreChildReaders();
    
    public MessageReader getNextChildReader();
    
    public QName getName();
    
    /**
     * Get the local name of the element this reader represents.
     * @return
     */
    public String getLocalName();

    /**
     * @return
     */
    public String getNamespace();

    /**
     * Get the XMLStreamReader. If you use this, you can't
     * use any of the other methods on the MessageReader.
     * It must be returned directly after the END_ELEMENT
     * tag.
     * 
     * @return
     */
    public XMLStreamReader getXMLStreamReader();

}
