package org.codehaus.xfire.message;

import java.lang.reflect.Constructor;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.codehaus.xfire.MessageContext;
import org.codehaus.xfire.XFireRuntimeException;
import org.codehaus.xfire.fault.XFireFault;
import org.codehaus.xfire.handler.AbstractHandler;
import org.codehaus.xfire.handler.EndpointHandler;
import org.codehaus.xfire.service.object.Invoker;
import org.codehaus.xfire.service.object.ObjectInvoker;
import org.codehaus.xfire.service.object.Operation;

/**
 * Handles java services.
 * 
 * @author <a href="mailto:dan@envoisolutions.com">Dan Diephouse </a>
 * @since Feb 18, 2004
 */
public class ObjectServiceHandler
    extends AbstractHandler
    implements EndpointHandler
{
    private static final Log logger = LogFactory.getLog(ObjectServiceHandler.class.getName());

    public static final String NAME = "java";

    public static final String RESPONSE_VALUE = "xfire.java.response";

    public static final String RESPONSE_PIPE = "xfire.java.responsePipe";

    private final Invoker invoker;
    
    private final Class bridgeClass;
    
    public ObjectServiceHandler()
    {
        this(new ObjectInvoker());
    }

    public ObjectServiceHandler(final Invoker invoker)
    {
        this.invoker = invoker;
        this.bridgeClass = null;
    }
    
    public ObjectServiceHandler(final Invoker invoker, final Class bridge)
    {
        this.invoker = invoker;
        this.bridgeClass = bridge;
    }

    /**
     * @see org.codehaus.xfire.handler.Handler#invoke(org.codehaus.xfire.MessageContext)
     */
    public void invoke(final MessageContext context)
        throws XFireFault
    {
        try
        {
            MessageBridge pipe = getMessageBridge(context);
            
            // Read in the parameters...
            final List params = pipe.read();

            // Don't read the operation in until after reading. Otherwise
            // it won't work for document style services.
            final Operation operation = pipe.getOperation();

            // invoke the service method...
            final Object value = invoker.invoke(operation.getMethod(), params.toArray(), context);

            context.setProperty(RESPONSE_VALUE, value);
            context.setProperty(RESPONSE_PIPE, pipe);
        }
        catch (XFireRuntimeException e)
        {
            logger.warn("Error invoking service.", e);
            throw new XFireFault("Error invoking service.", e, XFireFault.SENDER);
        }
    }

    /**
     * @return
     * @throws XFireFault 
     */
    protected MessageBridge getMessageBridge(MessageContext context) 
        throws XFireFault
    {
        if (bridgeClass != null)
        {
            try
            {
                Constructor constructor =
                    bridgeClass.getConstructor(new Class[] {MessageContext.class});
                
                return (MessageBridge) constructor.newInstance( new Object[] { context } );
            }
            catch (Exception e)
            {
                logger.error("Couldn't create message bridge.", e);
                throw new XFireFault("Couldn't create message bridge", e, XFireFault.RECEIVER);
            }
            
        }
        return MessageBridgeFactory.createMessageBridge(context);
    }

    public void writeResponse(final MessageContext context)
        throws XFireFault
    {
        final MessageBridge pipe = (MessageBridge) context.getProperty(RESPONSE_PIPE);
        final Object value = context.getProperty(RESPONSE_VALUE);

        pipe.write(new Object[] { value });
    }
}
