package org.codehaus.xfire.message;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.codehaus.xfire.MessageContext;
import org.codehaus.xfire.fault.XFireFault;
import org.codehaus.xfire.service.object.Operation;
import org.codehaus.xfire.service.object.Parameter;
import org.codehaus.xfire.util.DepthXMLStreamReader;

/**
 * Reads Document/Literal style messages.
 * 
 * @author <a href="mailto:dan@envoisolutions.com">Dan Diephouse</a>
 * @since Apr 7, 2004
 */
public class RPCEncodedBridge
    extends AbstractMessageBridge
{
    public RPCEncodedBridge(MessageContext context)
    {
        super(context);
	}
    
    public List read() 
    	throws XFireFault
    {
        List parameters = new ArrayList();
        LiteralReader reader = new LiteralReader(new DepthXMLStreamReader(getRequestReader()));
        
        if ( !reader.hasMoreChildReaders() )
            throw new XFireFault("There must be a method name element.", XFireFault.SENDER);
        
        MessageReader methodReader = reader.getNextChildReader();

        String opName = methodReader.getLocalName();
        Operation operation = getService().getOperation( opName );
        if (operation == null)
        {
            // Determine the operation name which is in the form of:
            // xxxxRequest where xxxx is the operation.
            int index = opName.indexOf("Request");
            if (index > 0)
            {
                operation = getService().getOperation( opName.substring(0, index) );
            }
        }

        setOperation( operation );

        Operation op = getOperation();

        if ( op == null )
        {
            throw new XFireFault("Invalid operation: "+opName, XFireFault.SENDER);
        }

        while( methodReader.hasMoreChildReaders() )
        {
            MessageReader cReader = methodReader.getNextChildReader();
            
            Parameter p =  op.getInParameter(cReader.getName());

            if (p == null)
            {
                throw new XFireFault("No such IN parameter: "+cReader.getName(), XFireFault.SENDER);
            }

            parameters.add(p.read(cReader, getContext()));
        }
        
        return parameters;
    }

    /**
     * @see org.codehaus.xfire.message.MessageBridge#write(java.lang.Object)
     */
    public void write(Object[] values)
    	throws XFireFault
    {
        String name = getOperation().getName() + "Response";
        LiteralWriter writer = new LiteralWriter( 
                getResponseWriter(), 
                name, 
                getService().getDefaultNamespace() );
        
        int i = 0;
        for(Iterator itr = getOperation().getOutParameters().iterator(); itr.hasNext();)
        {
            Parameter outParam = (Parameter) itr.next();
            outParam.write(values[i], writer, getContext());
            i++;
        }
        
        writer.close();
    }
}
