package org.codehaus.xfire.message;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.codehaus.xfire.MessageContext;
import org.codehaus.xfire.fault.XFireFault;
import org.codehaus.xfire.service.object.Operation;
import org.codehaus.xfire.service.object.Parameter;
import org.codehaus.xfire.util.DepthXMLStreamReader;

/**
 * Reads Document/Literal style messages.
 * 
 * @author <a href="mailto:dan@envoisolutions.com">Dan Diephouse</a>
 * @since Apr 7, 2004
 */
public class WrappedBridge
    extends AbstractMessageBridge
{
    public WrappedBridge(MessageContext context)
    {
        super(context);
	}
    
    public List read() 
    	throws XFireFault
    {
        List parameters = new ArrayList();
        LiteralReader reader = new LiteralReader(new DepthXMLStreamReader(getRequestReader()));
        
        if ( !reader.hasMoreChildReaders() )
            throw new XFireFault("There must be a method name element.", XFireFault.SENDER);
        
        MessageReader methodReader = reader.getNextChildReader();

        setOperation(getService().getOperation( methodReader.getLocalName() ));

        Operation op = getOperation();

        if (op == null)
        {
            throw new XFireFault("Invalid operation.", XFireFault.SENDER);
        }

        while(methodReader.hasMoreChildReaders())
        {
            MessageReader cReader = methodReader.getNextChildReader();
            
            Parameter p =  op.getInParameter(cReader.getName());

            if (p == null)
            {
                throw new XFireFault("Parameter " + cReader.getName() + " does not exist!", 
                                     XFireFault.SENDER);
            }

            parameters.add(p.read(cReader, getContext()));
        }
        
        return parameters;
    }

    /**
     * @see org.codehaus.xfire.message.MessageBridge#write(java.lang.Object)
     */
    public void write(Object[] values)
    	throws XFireFault
    {
        String name = getOperation().getName() + "Response";
        LiteralWriter writer = new LiteralWriter( 
                getResponseWriter(), 
                name, 
                getService().getDefaultNamespace() );
        
        int i = 0;
        for(Iterator itr = getOperation().getOutParameters().iterator(); itr.hasNext();)
        {
            Parameter outParam = (Parameter) itr.next();

            outParam.write(values[i], writer, getContext());
            i++;
        }

        writer.close();
    }
}
