package org.codehaus.xfire.type;

import java.util.Collection;

import javax.xml.namespace.QName;

import org.codehaus.xfire.type.basic.ArrayType;
import org.codehaus.xfire.type.basic.BeanType;
import org.codehaus.xfire.util.NamespaceHelper;

/**
 * A type mapping which automatically generates types
 * for java classes which are not registered, allowing
 * easy deployment of java services.
 * 
 * @author <a href="mailto:dan@envoisolutions.com">Dan Diephouse</a>
 * @since Feb 21, 2004
 */
public class AutoTypeMapping
    extends CustomTypeMapping
{
    public AutoTypeMapping( TypeMapping defaultTM )
    {
        super( defaultTM );
    }
    
    public AutoTypeMapping()
    {
        super();
    }
	
	/**
	 * @see org.codehaus.xfire.type.TypeMapping#getSerializer(java.lang.Class, javax.xml.namespace.QName)
	 */
	public Type getType( Class javaType, QName xmlType )
	{
        Type type = super.getType(javaType, xmlType);
        
        if ( type == null )
        {
            type = getType(javaType);
            
        	register( javaType, xmlType, type );
            
            type = super.getType(javaType, xmlType);
        }
        
        return type;
	}

    /**
     * Tries to determine a type class automatically from the type.
     * 
	 * @param javaType
	 * @return
	 */
	protected Type findTypeForClass(Class javaType)
	{
		if ( javaType.isArray() ||
             javaType.isAssignableFrom( Collection.class ) )
        {
			return new ArrayType();
        }
        else
        {
        	return new BeanType();
        }
	}

	/**
     * @see org.codehaus.xfire.type.TypeMapping#getType(java.lang.Class)
     */
    public Type getType(Class javaType)
    {
        Type type = super.getType(javaType);

        if ( type == null )
        {
            QName qname = createQName( javaType );
            register( javaType, qname, findTypeForClass( javaType ) );

            type = super.getType(javaType, qname);
        }
        
        return type;
    }

	/**
	 * @param javaType
	 * @return
	 * @throws ClassNotFoundException
	 */
	private QName createQName(Class javaType)
	{
        String clsName = javaType.getName();
        
        if (clsName.startsWith("[L"))
		{
			clsName = clsName.substring(2, clsName.length() - 1);
		}

        String ns = NamespaceHelper.makeNamespaceFromClassName(clsName, "http");
        
        String localName = null;
        
        if (javaType.isArray() ||
            javaType.isAssignableFrom( Collection.class ))
        {
            localName = "ArrayOf" + clsName.substring( clsName.lastIndexOf(".")+1 );
            
            // If this is an array of a primitive type, put the type
            // we're creating in the default namespace.
            if ( javaType.isArray() )
            {
                Type type = getType( javaType.getComponentType() );
                
                if ( type.isComplex() )
                {
                    ns = type.getSchemaType().getNamespaceURI();
                }
                else
                {
                    ns = getEncodingStyleURI();
                }
            }
        }
        else
        {
            localName = clsName.substring( clsName.lastIndexOf(".")+1 );
        }

        return new QName( ns, localName );
	} 
}
