package org.codehaus.xfire.type.basic;

import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.lang.reflect.Array;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.codehaus.xfire.MessageContext;
import org.codehaus.xfire.XFireRuntimeException;
import org.codehaus.xfire.fault.XFireFault;
import org.codehaus.xfire.message.MessageReader;
import org.codehaus.xfire.message.MessageWriter;
import org.codehaus.xfire.soap.SoapConstants;
import org.codehaus.xfire.type.Type;
import org.codehaus.xfire.util.NamespaceHelper;
import org.dom4j.DocumentFactory;
import org.dom4j.Element;
import org.dom4j.Namespace;
import org.dom4j.QName;

/**
 * An ArrayType.
 * 
 * @author <a href="mailto:dan@envoisolutions.com">Dan Diephouse</a>
 */
public class ArrayType
    extends Type
{
    public Object readObject(MessageReader reader, MessageContext context)
        throws XFireFault
    {
        try
        {
            Type compType = getComponentType();
            
            List values = new ArrayList();
            
            while ( reader.hasMoreChildReaders() )
            {
                Type type = getTypeMapping().getType( compType.getTypeClass() );
                
                values.add( type.readObject(reader.getNextChildReader(), context) );
            }
            
            return values.toArray( (Object[]) Array.newInstance( compType.getTypeClass(), values.size()) );
        }
        catch (IllegalArgumentException e)
        {
            throw new XFireRuntimeException("Illegal argument.", e);
        }
    }

    public void writeObject(Object object, MessageWriter writer, MessageContext context)
        throws XFireFault
    {
        if (object == null)
            return;

        try
        {
        	Object[] array = (Object[]) object;
            
            Type type = getComponentType();
            
            if ( type == null )
                throw new XFireRuntimeException( "Couldn't find type for " + type.getTypeClass() + "." );
            
            for ( int i = 0; i < array.length; i++ )
            {
                BeanInfo info = Introspector.getBeanInfo( type.getTypeClass(), Object.class );

                MessageWriter cwriter = writer.getChildWriter(type.getSchemaType().getLocalPart());
                type.writeObject( array[i], writer, context );
                cwriter.close();
            }
        }
        catch (IntrospectionException e)
        {
            throw new XFireRuntimeException("Couldn't introspect.", e);
        }
        catch (IllegalArgumentException e)
        {
            throw new XFireRuntimeException("Illegal argument.", e);
        }
    }

    public void writeSchema( Element root )
    {
        try
        {        
            Namespace xsdNs = root.getNamespaceForURI( SoapConstants.XSD );
            org.dom4j.QName complexQ = new org.dom4j.QName("complexType", xsdNs);   

            Element complex = root.addElement( complexQ );

            complex.addAttribute( "name", getSchemaType().getLocalPart() );

            org.dom4j.QName seqQ = new org.dom4j.QName("sequence", xsdNs); 
            Element seq = complex.addElement( seqQ );

            org.dom4j.QName elementQ = new org.dom4j.QName("element", xsdNs); 
            Element element = seq.addElement( elementQ );
            
            Type componentType = getComponentType();
            Namespace ns = NamespaceHelper.getNamespace( root, componentType.getSchemaType().getNamespaceURI() );

            QName typeDom4j = DocumentFactory.getInstance().createQName( componentType.getSchemaType().getLocalPart(),
                                                                         ns );
            
            String typeName = typeDom4j.getNamespacePrefix() + ":" + componentType.getSchemaType().getLocalPart();
            
            element.addAttribute("name", componentType.getSchemaType().getLocalPart());
            element.addAttribute("type", typeName);
            // TODO: Add config support for nillable
            element.addAttribute("nillable", "true");
 
            element.addAttribute("minOccurs", "0");
            element.addAttribute("maxOccurs", "unbounded");
            
        }
        catch (IllegalArgumentException e)
        {
            throw new XFireRuntimeException("Illegal argument.", e);
        }
    }
    
    /**
     * We need to write a complex type schema for Beans, so return true.
     * 
     * @see org.codehaus.xfire.type.Type#isComplex()
     */
    public boolean isComplex()
    {
        return true;
    }
    
    /**
     * @see org.codehaus.xfire.type.Type#getDependencies()
     */
    public Set getDependencies()
    {
        Set deps = new HashSet();
        
        deps.add( getTypeMapping().getType( getTypeClass().getComponentType() ) );
        
        return deps;
    }
    
    protected Type getComponentType()
    {
        Class compType = getTypeClass().getComponentType();
        
        return getTypeMapping().getType( compType );
    }
}
