package org.codehaus.xfire.wsdl11.builder;

import java.util.Collection;

import javax.wsdl.WSDLException;

import org.codehaus.xfire.AbstractXFireComponent;
import org.codehaus.xfire.service.Service;
import org.codehaus.xfire.service.object.ObjectService;
import org.codehaus.xfire.soap.SoapConstants;
import org.codehaus.xfire.transport.TransportManager;
import org.codehaus.xfire.wsdl.WSDLWriter;

/**
 * Create a WSDL document for a ObjectService.
 * 
 * @author <a href="mailto:dan@envoisolutions.com">Dan Diephouse</a>
 * @since Apr 7, 2004
 */
public class JavaWSDLBuilder
	extends AbstractXFireComponent
	implements WSDLBuilder
{
    private TransportManager manager;
    
    public JavaWSDLBuilder( TransportManager manager )
    {
        this.manager = manager;
    }

	/**
	 * @param s The service WSDL is being generated for.
	 * @return
	 */
	public WSDLWriter createWSDLWriter( Service s )
        throws WSDLException
	{
        ObjectService service = (ObjectService) s;
        
        Collection transports = manager.getTransports(service.getName());
	    
		if (service.getStyle().equals(SoapConstants.STYLE_WRAPPED) 
            && service.getUse().equals(SoapConstants.USE_LITERAL))
        {
            return new WrappedWSDL(service, transports);
        }
        else if (service.getStyle().equals(SoapConstants.STYLE_DOCUMENT) 
                 && service.getUse().equals(SoapConstants.USE_LITERAL))
        {
            return new DocumentWSDL(service, transports);
        }
        else if (service.getStyle().equals(SoapConstants.STYLE_RPC)
                 && service.getUse().equals(SoapConstants.USE_ENCODED))
        {
            return new RPCEncodedWSDL(service, transports);
        }
        else
        {
        	throw new UnsupportedOperationException( 
        	        "Service style/use combination is not supported: "
        	        + service.getStyle() + "/" + service.getUse() );
        }
	}
}
