package org.codehaus.xfire.wsdl11.builder;

import java.util.Collection;
import java.util.Iterator;

import javax.wsdl.Message;
import javax.wsdl.Part;
import javax.wsdl.WSDLException;
import javax.xml.namespace.QName;

import org.codehaus.xfire.XFireRuntimeException;
import org.codehaus.xfire.service.object.ObjectService;
import org.codehaus.xfire.service.object.Operation;
import org.codehaus.xfire.service.object.Parameter;
import org.codehaus.xfire.type.Type;
import org.dom4j.Element;
import org.dom4j.Namespace;

/**
 * Creates Document style WSDL documents for JavaServices.
 * 
 * @author <a href="mailto:dan@envoisolutions.com">Dan Diephouse</a>
 */
public class WrappedWSDL
    extends DocumentWSDL
{    

    public WrappedWSDL( ObjectService service, Collection transports ) throws WSDLException
    {
        super( service, transports );
    }

    private QName createResponseDocumentType(Operation op, Part part)
    {
        ObjectService service = (ObjectService) getService();
        
        String opName = op.getName() + "Response";

        Element schemaEl = createSchemaType( service.getDefaultNamespace() );
        Element element = schemaEl.addElement( elementQ );
        element.addAttribute( "name", opName );
        
        org.dom4j.QName complexQ = new org.dom4j.QName("complexType", xsdNs);
        Element complex = element.addElement( complexQ );

        if ( op.getOutParameters().size() > 0 )
        {
            Element sequence = createSequence( complex );
    
            writeParametersSchema(op.getOutParameters(), sequence);
        }

        return new QName( getService().getDefaultNamespace(), opName );
    }

    private QName createRequestDocumentType(Operation op, Part part)
    {
        ObjectService service = (ObjectService) getService();
        
        String opName = op.getName();// + "Request";

        Element schemaEl = createSchemaType( service.getDefaultNamespace() );
        Element element = schemaEl.addElement( elementQ );
        element.addAttribute( "name", opName );
        
        org.dom4j.QName complexQ = new org.dom4j.QName("complexType", xsdNs);
        Element complex = element.addElement( complexQ );
        
        if ( op.getInParameters().size() > 0 )
        {
            Element sequence = createSequence( complex );
    
            writeParametersSchema(op.getInParameters(), sequence);
        }

        return new QName( getService().getDefaultNamespace(), opName );
    }

    /**
     * @param op
     * @param sequence
     */
    private void writeParametersSchema(Collection params, Element sequence)
    {
        ObjectService service = (ObjectService) getService();
        
        for ( Iterator itr = params.iterator(); itr.hasNext(); )
        {
            Parameter param = (Parameter) itr.next();
            
            Class clazz = param.getTypeClass();
            QName pName = param.getName();
            Type type = service.getTypeMapping().getType(clazz);

            if (type == null)
                throw new XFireRuntimeException("Couldn't find type for parameter " + param.getName());
            
            addDependency( type );
            QName schemaType = type.getSchemaType();

            Namespace typeNS = getNamespace( type.getSchemaType().getNamespaceURI() );
            addNamespace( typeNS.getPrefix(), typeNS.getURI() );
            
            org.dom4j.QName outElementQ = new org.dom4j.QName("element", xsdNs);
            Element outElement = sequence.addElement( outElementQ );

            outElement.addAttribute("name", pName.getLocalPart() );
            outElement.addAttribute("type", typeNS.getPrefix() + ":" + type.getSchemaType().getLocalPart() );

            outElement.addAttribute("minOccurs", "1");
            outElement.addAttribute("maxOccurs", "1");
        }
    }

    private Element createSequence( Element complex )
    {
        org.dom4j.QName sequenceQ = new org.dom4j.QName("sequence", xsdNs);
        return complex.addElement( sequenceQ );
    }

	/**
	 * @see org.codehaus.xfire.wsdl11.builder.AbstractJavaWSDL#createInputParts(javax.wsdl.Message, org.codehaus.xfire.java.Operation)
	 */
	protected void createInputParts( Message req, Operation op )
	{
        Part part = getDefinition().createPart();
            
        QName typeQName = createRequestDocumentType(op, part);
        
        part.setName( "parameters" );
        part.setElementName( typeQName );
        
        req.addPart( part );
	}

	/**
	 * @see org.codehaus.xfire.wsdl11.builder.AbstractJavaWSDL#createOutputParts(javax.wsdl.Message, org.codehaus.xfire.java.Operation)
	 */
	protected void createOutputParts( Message req, Operation op )
	{
        // response message part
        Part part = getDefinition().createPart();

        // Document style service
        QName typeQName = createResponseDocumentType(op, part);
        part.setElementName( typeQName );
        part.setName( "parameters" );
        
        req.addPart( part );
	}
}
